@@ -1,25 +1,40 @@
 import numpy as np
 import pytest
-from probatus.utils import NotFittedError
 
 from probatus.binning import SimpleBucketer, QuantileBucketer, AgglomerativeBucketer, TreeBucketer
+from sklearn.exceptions import NotFittedError
+from sklearn.utils.validation import check_is_fitted
 
 
+@pytest.mark.filterwarnings("ignore:")
+def test_deprecations():
+    x = [1, 2, 1]
+    bins = 3
+    myBucketer = SimpleBucketer(bin_count=bins)
+    myBucketer.fit(x)
+    with pytest.deprecated_call():
+        myBucketer.counts
+
+    with pytest.deprecated_call():
+        myBucketer.boundaries
+    
+    
 def test_simple_bins():
     x = [1, 2, 1]
     bins = 3
     myBucketer = SimpleBucketer(bin_count=bins)
-    assert not myBucketer.fitted
+    with pytest.raises(NotFittedError):
+       myBucketer.compute([1,2])
+    
     myBucketer.fit(x)
-    assert myBucketer.fitted
-    assert len(myBucketer.counts) == bins
-    assert np.array_equal(myBucketer.counts, np.array([2, 0, 1]))
-    assert len(myBucketer.boundaries) == bins + 1
-    np.testing.assert_array_almost_equal(myBucketer.boundaries, np.array([1., 1.33333333, 1.66666667, 2.]))
+    assert len(myBucketer.counts_) == bins
+    assert np.array_equal(myBucketer.counts_, np.array([2, 0, 1]))
+    assert len(myBucketer.boundaries_) == bins + 1
+    np.testing.assert_array_almost_equal(myBucketer.boundaries_, np.array([1., 1.33333333, 1.66666667, 2.]))
     # test static method
     counts, boundaries = SimpleBucketer(bin_count=bins).simple_bins(x, bins)
-    assert np.array_equal(myBucketer.counts, counts)
-    np.testing.assert_array_almost_equal(myBucketer.boundaries, boundaries)
+    assert np.array_equal(myBucketer.counts_, counts)
+    np.testing.assert_array_almost_equal(myBucketer.boundaries_, boundaries)
     assert repr(myBucketer).startswith('SimpleBucketer')
 
 
@@ -28,17 +43,17 @@ def test_quantile_bins():
     random_state = np.random.RandomState(0)
     x = random_state.normal(0, 1, size=1000)
     myBucketer = QuantileBucketer(bin_count=bins)
-    assert not myBucketer.fitted
+    with pytest.raises(NotFittedError):
+       myBucketer.compute([1,2])
     myBucketer.fit(x)
-    assert myBucketer.fitted
-    assert len(myBucketer.counts) == bins
-    assert np.array_equal(myBucketer.counts, np.array([250, 250, 250, 250]))
-    assert len(myBucketer.boundaries) == bins + 1
-    np.testing.assert_array_almost_equal(myBucketer.boundaries, np.array([-3.0, -0.7, -0.1, 0.6, 2.8]), decimal=1)
+    assert len(myBucketer.counts_) == bins
+    assert np.array_equal(myBucketer.counts_, np.array([250, 250, 250, 250]))
+    assert len(myBucketer.boundaries_) == bins + 1
+    np.testing.assert_array_almost_equal(myBucketer.boundaries_, np.array([-3.0, -0.7, -0.1, 0.6, 2.8]), decimal=1)
     # test static method
     counts, boundaries = QuantileBucketer(bin_count=bins).quantile_bins(x, bins)
-    assert np.array_equal(myBucketer.counts, counts)
-    np.testing.assert_array_almost_equal(myBucketer.boundaries, boundaries)
+    assert np.array_equal(myBucketer.counts_, counts)
+    np.testing.assert_array_almost_equal(myBucketer.boundaries_, boundaries)
     # test inf edges
     counts, boundaries = QuantileBucketer(bin_count=bins).quantile_bins(x, bins, inf_edges=True)
     assert boundaries[0] == -np.inf
@@ -53,17 +68,17 @@ def test_agglomerative_clustering_new():
     x = [log_function(x) for x in np.arange(-1, 1, 0.01)]
     bins = 4
     myBucketer = AgglomerativeBucketer(bin_count=bins)
-    assert not myBucketer.fitted
+    with pytest.raises(NotFittedError):
+       myBucketer.compute([1,2])
     myBucketer.fit(x)
-    assert myBucketer.fitted
-    assert len(myBucketer.counts) == bins
-    assert np.array_equal(myBucketer.counts, np.array([24, 16, 80, 80]))
-    assert len(myBucketer.boundaries) == bins + 1
-    np.testing.assert_array_almost_equal(myBucketer.boundaries, np.array([0, 0.11, 0.59, 0.88, 0.99]), decimal=2)
+    assert len(myBucketer.counts_) == bins
+    assert np.array_equal(myBucketer.counts_, np.array([24, 16, 80, 80]))
+    assert len(myBucketer.boundaries_) == bins + 1
+    np.testing.assert_array_almost_equal(myBucketer.boundaries_, np.array([0, 0.11, 0.59, 0.88, 0.99]), decimal=2)
     # test static method
     counts, boundaries = AgglomerativeBucketer(bin_count=bins).agglomerative_clustering_binning(x, bins)
-    assert np.array_equal(myBucketer.counts, counts)
-    np.testing.assert_array_almost_equal(myBucketer.boundaries, boundaries)
+    assert np.array_equal(myBucketer.counts_, counts)
+    np.testing.assert_array_almost_equal(myBucketer.boundaries_, boundaries)
     assert repr(myBucketer).startswith('AgglomerativeBucketer')
 
 
@@ -76,8 +91,8 @@ def test_compute():
         assert myBucketer.compute(x_new)
     myBucketer.fit(x)
     assert len(myBucketer.compute(x_new)) == bins
-    np.testing.assert_array_equal(myBucketer.counts, myBucketer.compute(x_new))
-    np.testing.assert_array_equal(myBucketer.counts, myBucketer.fit_compute(x_new))
+    np.testing.assert_array_equal(myBucketer.counts_, myBucketer.compute(x_new))
+    np.testing.assert_array_equal(myBucketer.counts_, myBucketer.fit_compute(x_new))
     x_new = x + 100
     np.testing.assert_array_equal(np.array([0, 0, 0, 0, 0]), myBucketer.compute(x_new))
     x_new = x - 100
@@ -115,25 +130,26 @@ def test_tree_bucketer():
 
     myTreeBucketer = TreeBucketer(inf_edges=False,max_depth=3,min_samples_leaf=10,random_state=42)
 
-    assert not myTreeBucketer.fitted
+    with pytest.raises(NotFittedError):
+       myTreeBucketer.compute([1,2])
 
     myTreeBucketer.fit(x,y)
 
-    assert all(myTreeBucketer.counts==np.array([21, 15, 14]))
+    assert all(myTreeBucketer.counts_==np.array([21, 15, 14]))
     assert myTreeBucketer.bin_count==3
-    assert all(myTreeBucketer.boundaries - np.array([0.0, 4.1, 7.1, 9.8])<0.01)
+    assert all(myTreeBucketer.boundaries_ - np.array([0.0, 4.1, 7.1, 9.8])<0.01)
 
     # If infinite edges is False, it must get the edges of the x array
-    assert myTreeBucketer.boundaries[0] == 0
-    assert myTreeBucketer.boundaries[-1] == 9.8
+    assert myTreeBucketer.boundaries_[0] == 0
+    assert myTreeBucketer.boundaries_[-1] == 9.8
 
 
     myTreeBucketer = TreeBucketer(inf_edges=True,max_depth=3, min_samples_leaf=10, random_state=42)
 
     myTreeBucketer.fit(x, y)
     # check that the infinite edges is True, then edges must be infinite
-    assert myTreeBucketer.boundaries[0] == -np.inf
-    assert myTreeBucketer.boundaries[-1] == +np.inf
+    assert myTreeBucketer.boundaries_[0] == -np.inf
+    assert myTreeBucketer.boundaries_[-1] == +np.inf
 
 
 def test_tree_bucketer_dependence():
@@ -151,10 +167,10 @@ def test_tree_bucketer_dependence():
 
     # Test that the counts per bin never drop below min_samples_leaf
     myTreeBucketer = TreeBucketer(inf_edges=False, max_depth=6, min_samples_leaf=100, random_state=42).fit(x, y)
-    assert all(myTreeBucketer.counts >= myTreeBucketer.tree.min_samples_leaf)
+    assert all(myTreeBucketer.counts_ >= myTreeBucketer.tree.min_samples_leaf)
 
     myTreeBucketer = TreeBucketer(inf_edges=False, max_depth=6, min_samples_leaf=200, random_state=42).fit(x, y)
-    assert all(myTreeBucketer.counts >= myTreeBucketer.tree.min_samples_leaf)
+    assert all(myTreeBucketer.counts_ >= myTreeBucketer.tree.min_samples_leaf)
 
     # Test that if the leaf is set to the number of entries,it raises an Error
     myTreeBucketer = TreeBucketer(inf_edges=False, max_depth=6, min_samples_leaf=x.shape[0], random_state=42)
@@ -168,6 +184,6 @@ def test_tree_bucketer_dependence():
         .fit(x, y)
     )
     assert myTreeBucketer.bin_count ==1
-    assert all(myTreeBucketer.counts >= myTreeBucketer.tree.min_samples_leaf)
+    assert all(myTreeBucketer.counts_ >= myTreeBucketer.tree.min_samples_leaf)
 
 
